<div id="user_dialog" class="modal hide {{ 'no_queries' if not show_queries else '' }}"
  tabindex="-1" role="dialog">
  <div class="modal_header">
    <button type="button" class="close glyphicon glyphicon-minus" style='left: -20px'
      data-dismiss="modal" aria-hidden="true"></button>
  </div>
  <div class="modal_body">
    {% if show_queries %}
      <div class="">
        <div id="query_queue" class="collapsed needsclick" style="text-align: left">

        <ul class="nav nav-tabs">
          <li class="active"><a href="#query_list" data-toggle="tab">Recent Queries</a></li>
        </ul>

        <div class="tab-content">
          <div id="query_list" class="clearfix query_list tab-pane active">
          </div>

          <div id="saved_queries" class="clearfix saved_queries tab-pane">
          </div>

          <div id="shared_queries" class="clearfix shared_queries tab-pane">
          </div>
        </div>
      </div>
    </div>
    {% else %}
      <div style='height: 100px; padding: 20px;'>
      <h3>Welcome to snorkelite</h3>
      <hr />
      <h5>You are logged in as <a href='/user'><b>{{ user.email }}</b></a></h5>
      </div>
    {% endif %}

  <div class="modal-footer">
    <a class="btn btn-link lfloat logout" href="/logout">(Logout)</a>
    <button class="btn" data-dismiss="modal" aria-hidden="true">Close</button>
  </div>
</div>
</div>

