.modal {
  overflow: auto;
  max-height: 550px;
}

.modal-footer {
  position: absolute;
  left: 0px;
  right: 0px;
  bottom: 0px;
}

.modal.no_queries {
  max-height: 250px;
}

.query_list {
  height: 420px;
}

.logout {
  float: left;
  position: absolute;
  left: 0;
  color: black;
}

@media #{$mobile} {
  .modal {
    max-width: 550px;
    width: 80%;
    left: 10%;
    right: 10%;
    margin-left: 0px;
  }
}
