<% set_default("icon", "") %>
<% set_default("results", "") %>
<% set_default("username", null) %>
<% set_default("query", { parsed: {}}) %>
<div class="query_tile">
  <div class="query_graph mll">

    <div width="100%" background-color="#000">

      <div class='titlehover'>
        <a href='/query?h=<%= query.hashid %>&table=<%= query.table %>'>
          <span class='title'>

            <% if (query.title) { %>
              <%= query.title %>
            <% } else { %>
              <%= (query.parsed.view  + " view") %>
            <% } %>

          </span>
        </a>

      </div>


      <div class="pll">
        <small class="fsi">
        <% if (query.updated != query.created) { %>
          <span class="updated" style='display: none;'>updated
            <span class="timestamp"></span>,
          </span>
        <% } %>

        <% if (query.username) { %>
            created by <%= query.username %>
        <% } %>
          <span class="created timestamp"> </span>

        </small>
      </div>
    </div>





    <div class="row-fluid pll">


      <div>
        <div class="span6">
          <b>Dataset:</b> <%= query.parsed.table %>

          <% if (query.title) { %>
            <div>
              <b>View:</b> <%= query.parsed.view %>
            </div>
          <% } %>


          <div><% if (query.parsed.dims.length) { %> <b>Grouped by:</b> <%= query.parsed.dims.join(',') %> <% } %></div>
          <div><% if (query.parsed.cols.length) { %> <b>Fields:</b> <%= query.parsed.cols.join(',') %> <% } %></div>

        </div>

        <div class="span6">
          <% if (query.parsed.start) { %> <b>From:</b> <%= query.parsed.start %> <% } %>
          <% if (query.parsed.end) { %> &ndash; <%= query.parsed.end %> <% } %>
          <% if (query.parsed.compare_str) { %>
            <br />
            <b>Compared against:</b> <%= query.parsed.compare_str %>
          <% } %>
        </div>


      </div>

      <div class="span12" style='min-height: inherit'>

        <% if (query.parsed.filters && _.isArray(query.parsed.filters.query || {}) && query.parsed.filters.query.length) { %>
          <div class="span6">
            <b>Filters</b>
            <% _.each(query.parsed.filters.query, function(arr) { %>
              <%
                var col = arr[0];
                var op = arr[1];
                var val = arr[2];



              %>
              <div><small class="pll">
                <%= col %>: <%= op %> <%= val %>

              </small></div>
            <% }) %>
          </div>
        <% } %>

        <% if (query.parsed.filters && query.parsed.filters.compare && query.parsed.filters.compare.length > 0) { %>
          <div class="span6">
            <b>Compare Filters</b>
            <% _.each(query.parsed.filters.compare, function(arr) { %>
              <%
                var col = arr[0];
                var op = arr[1];
                var val = arr[2];



              %>
              <div><small class="pll">
                <%= col %>: <%= op %> <%= val %>

              </small></div>
            <% }) %>
          </div>
        <% } %>
      </div>
    </div>
  </div>

</div>
