$mobile:   "only screen and (min-width: 200px) and (max-width: 599px)";
$tablet:   "only screen and (min-width: 600px) and (max-width: 767px)";
$device:   "only screen and (min-width: 200px) and (max-width: 767px)";
$netbook:   "only screen and (min-width: 768px) and (max-width: 1199px)";
$desktop:   "only screen and (min-width: 1200px) and (max-width: 1899px)";
$computer:   "only screen and (min-width: 768px)";
$large_screen:   "only screen and (min-width: 1900px)";

