thead {
  text-align: left;

}

.table td, .table th {
  font-size: 80%;
  border: 0px;
}

.table-striped > tbody > tr:nth-child(2n+1) > td,
.table-striped > tbody > tr:nth-child(2n+1) > th {
   background-color: #efefef;
}

.table td {
  max-width: 400px;
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
  border-right: 1px solid lightgray;
}

.table th {
  border-top: 0px;
  border-bottom: 0px;
}

.table td:last-child {
  border-right: 0px;
}


table thead { font-weight: bold; }

table.table-bordered { border: 0px; }

.delta {
  float: left;
  font-color: #888;
  font-size: 80%; }

.throbber {
  font-size: 24px;
  margin-top: 240px;
  text-align: center; }

/* Table formatting helpers */
.compare_cell {
  border-top: 1px dashed #aaa;
  margin: 5px;
  padding: 0px;
  text-align: right; }

.value_cell {
  margin: 5px;
  text-align: right; }

