.pageheader h3 {
  margin-top: 0.5em !important;
}

.pageheader .superscript {
  top: 0px !important;
}

.container-fluid, .container {
  margin: auto;
  margin-left: 0px;
  margin-top: 60px;

  .login_form, .password_form {
    width: 300px;
    margin: auto;
    margin-top: 100px;

    .btn {
      width: 100%;
      padding-left: 0px;
      padding-right: 0px;
    }

    label {
      display: inline-block;
    }

    input[type=text],
    input[type=password] {
      width: 90%;
      padding: 10px 5px;
      margin-top: 5px;
    }

    input[type=checkbox] {
      float: right;
      margin-right: 30px;
    }
  }
}

.centered {
  text-align: center;
}

fieldset {
  border: 0px;
}

.flashes {
  margin-left: -40px;
  text-align: center;
  li {
    list-style: none;
  }

}
