& {
  max-height: 600px;
}

.modal {
  border-radius: 5px;

  .modal-header {
    display: block;

  }

  .modal-footer {
    display: none;
  }


  .modal-header button {
    float: right;
    margin-top: -15px;
    clear: right;
  }
}

