
body {
  font-size: 15px;
}

.h1, h2, h3, h4, h5, h6 {
  margin: 0px;
}

.pageheader .selector {
  height: 30px;
  margin-left: 10px;
  margin-top: 5px;
}

.pageheader {
  background-color: #003;
  width: 100%;
  height: 50px;
  color: white;
  position: fixed;
  left: 0px;
  top: 0px;
  z-index: 10;

  select {
    width: 148px;
  }

  .superscript {
    font-size: 60%;
    margin-left: 2px;
    position:relative;
    top: 5px;
    vertical-align: top;
    font-weight: bold;

  }

  a {
    color: white;
    text-decoration: none;

    &:hover {
      color: lightgray;;
    }
  }

  h3 {
    font-size: 1.5em;
    margin-top: 0.3em;
    font-weight: bold;
    margin-left: 20px;
    margin-right: 10px;

  }

  h4 {
    font-size: 1em;
    margin-top: 1em;
    font-weight: normal;
    margin-left: 20px;
    margin-right: 20px;

  }
}

.body_spacer {
  height: 60px;
}

.tablesorter{
   width:100%;
   }


.table {
  display: table;
}

.snorkel_icon {
  width: 32px;
  height: 32px;
  margin-left: -5px;
  margin-top: 5px;
}
