
body {
  overflow-y: scroll;
  padding-bottom: 50px;
}

.container-fluid {
  margin-top: 60px;
}

.datasets {
  margin: auto;
  width: 600px;

}

.hidden_datasets {
  opacity: 0.5;
}

.hidden_datasets,
.superset {
  padding-left: 20px;

  h1, h2 {
    margin-left: -20px;
    margin-bottom: 10px;
    margin-top: 0px;
  }

}

.superset,.hidden_datasets {
  h1, h2 {
    cursor: pointer;
  }

  h1:hover, h2:hover {
    text-decoration: underline;
  }
}

.superset,.hidden_datasets {
  .dataset_tile {
    display: none;
  }
}

.superset.active,.hidden_datasets.active {
  .dataset_tile {
    display: block;
  }
}

.editlink {
  padding-top: 5px;
}

.status {
  font-family: monospace;
  padding-right: 5px;
  text-decoration: none !important;
  font-size: 22px;
}
