{{ add_stylesheet("login.css") }}

{% extends "layout.html" %}

{% block body %}
{% from "security/_macros.html" import render_field_with_errors, render_field %}
{% include "security/_messages.html" %}
<div class='container-fluid'>
  <h1 class='centered'>{{ _('Change password') }}</h1>
  <form class='password_form' action="{{ url_for_security('change_password') }}" method="POST" name="change_password_form">
    {{ change_password_form.hidden_tag() }}
    {{ render_field_with_errors(change_password_form.password) }}
    {{ render_field_with_errors(change_password_form.new_password) }}
    {{ render_field_with_errors(change_password_form.new_password_confirm) }}
    {{ render_field(change_password_form.submit) }}
  </form>
</div>
{% endblock %}
