<%= set_default("title", "") %>
<%= set_default("description", "") %>
<%= set_default("show_timestamp", true) %>

<% if (show_timestamp) { %>
  <div class="alert alert-info clearfix">
    <div class="mll mtm pull-left">
      This query is showing historic results from <span class='timestamp'> </span>
    </div>


  </div>
<% } %>

<div class='clearfix mtl'>
  <small class='lfloat mtl'>
    <div class='count'> </div>
    <div class='weight'> </div>
  </small>

  <small class='rfloat'>
    <span class='query_range_label mtl'>start:</span>
    <span class=''>
      <b>
        <% var d = new Date(query.parsed.start_ms) %>
        <span style='width: 80px; display: inline-block'>
          <%= d.toLocaleDateString() %>
        </span>
        <span style='width: 80px; text-align: right; margin-right: 10px'>
          <%= d.toLocaleTimeString('en-US', { hour12: false }) %>
        </span>
      </b>
    </span><br />

    <span class='query_range_label'>end:</span>
    <span class=''>
      <% var d = new Date(query.parsed.end_ms) %>
      <b>
        <span style='width: 80px; display: inline-block'>
          <%= d.toLocaleDateString() %>
        </span>
        <span style='width: 80px; text-align: right; margin-right: 10px'>
          <%= d.toLocaleTimeString('en-US', { hour12: false }) %>
        </span>
      </b>
    </span>
  </small>


</div>


<h3> <%= title %> </h3>
<p> <%= description %> </p>

<% if (title || description) { %>
  <hr />
<% } %>
