{{ add_stylesheet("login.css") }}

{% extends "layout.html" %}
{% from "security/_macros.html" import render_field_with_errors, render_field %}

{% block body %}
{% include "security/_messages.html" %}
<div class='container-fluid'>
  <div class='login_form'>
    <form action="{{ url_for_security('login') }}" method="POST" name="login_user_form" class="login_form">
      <fieldset>
        {{ login_user_form.hidden_tag() }}
        {{ render_field_with_errors(login_user_form.email) }}
        {{ render_field_with_errors(login_user_form.password) }}
        {{ render_field_with_errors(login_user_form.remember) }}
        {{ render_field(login_user_form.submit) }}
        {% if (google_auth) %}
          <a href="{{url_for('google.login')}}">Sign in With Google</a>
        {% endif %}
      </fieldset>

    </form>
  </div>
  {% include "security/_menu.html" %}
</div>
  {% endblock %}

