slides = [
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-24-27.png",
    "caption": "",
    "title": "Default view"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-26-49.png",
    "caption": "",
    "title": "Table View"
  },
  {
    "src": "/static/images/screens/gif/table_to_time2.gif",
    "caption": "",
    "title": "Table -> Time"
  },
  {
    "src": "/static/images/screens/gif/time_to_table.gif",
    "caption": "",
    "title": "Time -> Table"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-30-50.png",
    "caption": "",
    "title": "Distribution View (Moments)"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_11-53-01.png",
    "caption": "",
    "title": "Distribution View (Graphs)"
  },
  {
    "src": "/static/images/screens/gif/table_to_dist2.gif",
    "caption": "",
    "title": "Table -> Grouped Dist View"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-04-30_18-50-20.png",
    "caption": "",
    "title": "Overview"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-04-30_18-51-11.png",
    "caption": "",
    "title": "Overview (cont'd)"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-25-05.png",
    "caption": "",
    "title": "Time Series"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-25-43.png",
    "caption": "",
    "title": "Stacked Time Series"
  },
  {
    "src": "/static/images/screens/gif/forecasting2.gif",
    "caption": "",
    "title": "Time series forecasting"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-32-51.png",
    "caption": "",
    "title": "Anomaly detection"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-04-30_18-51-55.png",
    "caption": "",
    "title": "Bar Charts"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-27-57.png",
    "caption": "",
    "title": "Timelines"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-29-37.png",
    "caption": "",
    "title": "Scatter plots"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-32-23.png",
    "caption": "",
    "title": "Scatter plots (cont'd)"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-04-30_18-52-23.png",
    "caption": "",
    "title": "Scatter plots (cont'd)"
  },
  {
    "src": "/static/images/screens/Screenshot_2016-08-31_07-33-23.png",
    "caption": "",
    "title": "Samples"
  },
#  {
#    "src": "/static/images/screens/1264245_10201979227905947_1254692173_o.jpg",
#    "caption": "",
#    "title": "JSON API"
#  }
]
