{% extends "layout.html" %}

{% block body %}

  {% if error %}
    {{ error }}
  {% else %}
    <div class='container-fluid'>

      <div class='row'>
        <div class='sidebar_pane active'>
            {{ sidebar }}
        </div>

        <div class='results_pane'>
            <div class='results' id="query_content">
              {{ viewarea }}
            </div>
        </div>
      </div>

    </div>

    <div class='row'>
      <div id="mobile_buttons">
        <div class='btn btn-primary go rfloat'>Go</div>
        <div class='btn btn-secondary toggle_viewarea'>Show Results</div>
      </div>
    </div>

    <div id='status_bar' class="clearfix">
      <small>
        <span href="#" class="timing"> </span>
        <a href="#" class="details hidden"></a>
        <a href="http://github.com/logv/snorkel/issues/new" id="report_bug">see a bug? let us know!</a>
      </small>
    </div>
  {% endif %}
{% endblock %}
