<% set_default("type", null); %>
<% set_default("name", null); %>
<% set_default("formatter", null); %>

<div class="table_popover no-select">

  <% if (type === "integer") { %>
    <a class="overview">
      View column distribution
    </a>

    <hr />

    <a class="view" data-view="dist">
      View cell distribution
    </a>

    <a class="view" data-view="time">
      View over time
    </a>
  <% } %>

  <a class="view" data-view="time_count">
    View count over time
  </a>

  <a class="view" data-view="samples">
    View samples like this row
  </a>



  <% if (type === "string") { %>
    <hr />

    <a class="string option" data-op="$re">
      Add equals filter
    </a>
    <a class="string option" data-op="$nre">
      Add !equals filter
    </a>
  <% } %>

  <% if (type === "integer") { %>

    <hr />

    <a class="integer option" data-op="$gt">
      Add greater than filter
    </a>
    <a class="integer option" data-op="$lt">
      Add less than filter
    </a>
    <a class="integer option" data-op="$ne">
      Add not equals filter
    </a>
  <% } %>

</div>
